import os
import sys
import collections

KeyItem = collections.namedtuple("KeyItem", "redis key type ttl value")

REDIS_TEXT = """\
Db: {db}
Address: {addr}
Client ID: {id}
Client Name: {name}"""

def redis_str(redis):
    info = redis.connection_pool.connection_kwargs
    db = info["db"]
    cid = redis.client_id()
    cname = redis.client_getname()
    if "path" in info:  # unix socket
        addr = info["path"]
    elif "host" in info:
        addr = "{}:{}".format(info["host"], info["port"])
    else:
        addr = "???"
    ctext = "{} - {}".format(cid, cname) if cname else str(cid)
    text = "DB {} @ {} ({})".format(db, addr, ctext)
    long_text = REDIS_TEXT.format(db=db, addr=addr, id=cid, name=cname or "---")
    return text, long_text


def redis_value(redis, key, dtype=None):
    dtype = dtype or redis.type(key)
    value = None
    if dtype == "string":
        value = redis.get(key)
    elif dtype == "hash":
        value = redis.hgetall(key)
    elif dtype == "list":
        value = redis.lrange(key, 0, -1)
    elif dtype == "set":
        value = redis.smembers(key)
    return value


def redis_key_split(key, chars="."):
    result, curr = [], ""
    for char in key:
        if str(char) in chars:
            result.append(curr)
            curr = ""
        curr += str(char)
    if curr:
        result.append(curr)
    return result


__startup_cwd = os.getcwd()


def restart():
    if "linux" in sys.platform.lower():
        with open("/proc/{0}/cmdline".format(os.getpid())) as f:
            args = [arg for arg in f.read().split("\x00") if arg]
            executable = args[0]
    else:
        args = sys.argv[:]
        executable = sys.executable

    os.chdir(__startup_cwd)
    os.execvp(executable, args)



from .qt import QPainter, QPoint, QFont, QColor


def add_char_pixmap(pixmap, char, size=14, weight=QFont.Bold):
    if len(char) > 1:
        raise ValueError("Only one char is allowed")
    painter = QPainter(pixmap)
    painter.setFont(QFont("Times", size, weight))
    painter.setPen(QColor(255, 0, 0))
    w, h = pixmap.width(), pixmap.height()
    p = QPoint(w - size, h - 2)
    painter.drawText(p, char)


if __name__ == "__main__":
    from .qt import QApplication, QLabel, QIcon

    app = QApplication([])
    i = QIcon.fromTheme("list-add")
    p = i.pixmap(128, 128)
    add_char_pixmap(p, "S")
    lbl = QLabel()
    lbl.setPixmap(p)
    lbl.show()
    app.exec_()



import os
import logging
import functools
import collections

from .qt import (
    QMainWindow,
    QApplication,
    QTreeWidgetItem,
    QToolButton,
    QMessageBox,
    QIcon,
    QFont,
    QMenu,
    Signal,
    ui_loadable,
)
from .dialog import OpenRedisDialog
from .util import redis_str, redis_key_split
from .util import KeyItem as Item
from .redis import QRedis, ConnectionError


_this_dir = os.path.dirname(__file__)
_res_dir = os.path.join(_this_dir, "images")
_redis_icon = os.path.join(_res_dir, "redis_logo.png")
_key_icon = os.path.join(_res_dir, "key.png")


def fill_item(item, data):
    result = {}
    for key, value in data.items():
        child = KeyItem(key, value, item)
        children = value["children"]
        result[value["key"]] = child
        result.update(fill_item(child, children))
    return result


class RedisItem(QTreeWidgetItem):

    SplitChars = ".:"

    def __init__(self, redis, filter=None, split_by=SplitChars, parent=None):
        name, _ = redis_str(redis)
        super(RedisItem, self).__init__(parent, [name])
        self.__filter = filter
        self.__split = split_by
        self.key_items = {}
        self.setIcon(0, QIcon(_redis_icon))
        self.redis = redis
        redis.keyRenamed.connect(self.__on_key_renamed)
        redis.keysDeleted.connect(self.__on_keys_deleted)

    def toolTip(self, column):
        return redis_str(self.redis)[1]

    @property
    def filter(self):
        return self.__filter

    @filter.setter
    def filter(self, filter):
        self.__filter = filter
        self.update()

    def update(self):
        self.takeChildren()
        root_children = collections.OrderedDict()
        filt = self.filter or "*"
        for key in sorted(self.redis.keys(filt)):
            children = root_children
            sub_key, sub_keys = "", redis_key_split(key, self.__split)
            for i, name in enumerate(sub_keys):
                sub_key += name
                key_name = name[1:] if i else name
                item = children.setdefault(
                    key_name, dict(type="", children=collections.OrderedDict())
                )
                children = item["children"]
                item["name"] = key_name
                item["key"] = sub_key
                item["has_value"] = False
                item["has_value"] = True
        self.key_items = fill_item(self, root_children)

    def __on_key_renamed(self, old_item, new_item):
        old_key, new_key = old_item.key, new_item.key
        if not old_key:
            return
        old_sub_keys = redis_key_split(old_key, self.__split)
        new_sub_keys = redis_key_split(new_key, self.__split)
        parent_changed = old_sub_keys[:-1] != new_sub_keys[:-1]
        item = self.key_items.pop(old_key)
        if parent_changed:
            self.update()
        else:
            label = new_sub_keys[-1][1:] if len(new_sub_keys) > 1 else new_sub_keys[-1]
            self.key_items[new_key] = item
            item.rename(new_key, label)

    def __on_keys_deleted(self):
        self.update()

    @property
    def redis_item(self):
        return self


class KeyItem(QTreeWidgetItem):
    def __init__(self, key, data, parent):
        super(KeyItem, self).__init__(parent, [key])
        icon = QIcon(_key_icon) if data["has_value"] else QIcon.fromTheme("folder")
        self.setIcon(0, icon)
        self.setFont(3, QFont("Monospace"))
        self.__data = data
        self.root_item = self.__calc_root()

    def __calc_root(self):
        root = self
        while root.parent():
            root = root.parent()
        return root

    def rename(self, key, label):
        self.__data["key"] = key
        self.setText(0, label)

    @property
    def key(self):
        return self.__data.get("key")

    @property
    def has_value(self):
        return self.__data.get("has_value", False)

    @property
    def redis(self):
        return self.root_item.redis

    @property
    def redis_item(self):
        return self.root_item


@ui_loadable
class RedisTree(QMainWindow):

    addKey = Signal(object)
    selectionChanged = Signal(object)

    def __init__(self, parent=None):
        super(RedisTree, self).__init__(parent)
        self.load_ui()
        self.__selected_items = dict(
            all_items=[], key_items=[], db_items=[], db_keys={}
        )
        ui = self.ui
        header = ui.tree.header()
        header.resizeSection(0, 220)
        ui.open_db_action.setIcon(QIcon(_redis_icon))
        add_menu = QMenu("Add")
        ui.add_string_action = add_menu.addAction("string")
        ui.add_list_action = add_menu.addAction("list")
        ui.add_set_action = add_menu.addAction("set")
        ui.add_hash_action = add_menu.addAction("hash")
        add_button = QToolButton()
        add_button.setMenu(add_menu)
        add_button.setPopupMode(QToolButton.InstantPopup)
        add_button.setIcon(QIcon.fromTheme("list-add"))
        ui.add_key_action = ui.db_toolbar.insertWidget(ui.remove_key_action, add_button)
        ui.add_key_action.setEnabled(False)

        ui.tree.itemSelectionChanged.connect(self.__on_item_selection_changed)
        self.selectionChanged.connect(self.__on_selection_changed)
        ui.add_string_action.triggered.connect(
            functools.partial(self.__on_add_key, "string")
        )
        ui.add_list_action.triggered.connect(
            functools.partial(self.__on_add_key, "list")
        )
        ui.add_set_action.triggered.connect(functools.partial(self.__on_add_key, "set"))
        ui.add_hash_action.triggered.connect(
            functools.partial(self.__on_add_key, "hash")
        )

        ui.remove_key_action.triggered.connect(self.__on_remove_key)
        ui.touch_key_action.triggered.connect(self.__on_touch_key)

        ui.open_db_action.triggered.connect(self.__on_open_db)
        ui.close_db_action.triggered.connect(self.__on_close_db)
        ui.update_db_action.triggered.connect(self.__on_update_db)
        ui.flush_db_action.triggered.connect(self.__on_flush_db)

    def contextMenuEvent(self, event):
        pass

    @property
    def selected_items(self):
        return self.__selected_items

    def __update_selected_items(self):
        items = self.ui.tree.selectedItems()
        key_items = [item for item in items if isinstance(item, KeyItem)]
        db_items = [item for item in items if isinstance(item, RedisItem)]
        db_keys = collections.defaultdict(list)
        for key_item in key_items:
            db_keys[key_item.redis].append(key_item.key)
        si = self.__selected_items
        si["all_items"] = items
        si["key_items"] = key_items
        si["db_items"] = db_items
        si["db_keys"] = db_keys
        return si

    def __on_item_selection_changed(self):
        selected = self.__update_selected_items()
        self.selectionChanged.emit(selected)

    def __on_selection_changed(self, selected):
        ui = self.ui
        all_items = selected["all_items"]
        key_items = selected["key_items"]
        db_items = selected["db_items"]
        n_items, n_keys, n_dbs = map(len, (all_items, key_items, db_items))
        ui.add_key_action.setEnabled(n_items == 1)
        ui.remove_key_action.setEnabled(n_keys > 0)
        ui.persist_key_action.setEnabled(n_keys > 0)
        ui.touch_key_action.setEnabled(n_keys > 0)
        ui.copy_key_action.setEnabled(n_keys > 0)
        ui.flush_db_action.setEnabled(not n_keys and n_dbs)
        ui.update_db_action.setEnabled(n_items > 0)
        ui.close_db_action.setEnabled(n_dbs > 0)

    def __on_open_db(self):
        try:
            redis = OpenRedisDialog.create_redis()
            if redis:
                self.add_redis(redis)
        except Exception as error:
            QMessageBox.warning(self, "Connection error!", repr(error))

    def __on_close_db(self):
        pass

    def __on_flush_db(self):
        pass

    def __on_update_db(self):
        db_items = []
        for item in self.selected_items["all_items"]:
            db_items.append(item.redis_item)
        for db_item in db_items:
            db_item.update()

    def __on_swap_db(self):
        pass

    def __on_touch_key(self):
        selected = self.selected_items
        db_items = selected["db_items"]

    def __on_add_key(self, dtype):
        tree_item = self.ui.tree.selectedItems()[0]
        value = None
        if dtype == "string":
            value = ""
        elif dtype == "list":
            value = []
        elif dtype == "set":
            value = set()
        elif dtype == "hash":
            value = {}
        item = Item(tree_item.redis, "", dtype, -1, value)
        self.addKey.emit(item)

    def __on_remove_key(self):
        selected = self.selected_items
        key_items, db_keys = selected["key_items"], selected["db_keys"]
        for db, keys in db_keys.items():
            db.delete(*keys)
        self.ui.tree.clearSelection()

    def __on_update_item(self, item, column=None):
        try:
            item.update()
        except ConnectionError as ce:
            QMessageBox.critical(self, "Connection Error", str(ce))

    def add_redis(self, db):
        item = RedisItem(db)
        self.ui.tree.addTopLevelItem(item)
        try:
            item.update()
        except ConnectionError:
            pass
        item.setExpanded(True)


def main():
    import sys
    import argparse

    parser = argparse.ArgumentParser(description="QRedis GUI")
    parser.add_argument("--host", help="Server host name")
    parser.add_argument("-p", "--port", help="Server port", type=int)
    parser.add_argument("-s", "--sock", help="unix server socket")
    parser.add_argument("-n", "--db", help="Database number")
    parser.add_argument(
        "--log-level",
        default="WARNING",
        help="log level",
        choices=["CRITICAL", "ERROR", "WARNING", "INFO", "DEBUG"],
    )
    args = parser.parse_args()

    fmt = "%(asctime)-15s %(levelname)-5s %(name)s: %(message)s"
    level = getattr(logging, args.log_level.upper())
    logging.basicConfig(format=fmt, level=level)

    kwargs = {}
    if args.host is not None:
        kwargs["host"] = args.host
    if args.port is not None:
        kwargs["port"] = args.port
    if args.sock is not None:
        kwargs["unix_socket_path"] = args.sock
    application = QApplication(sys.argv)
    window = RedisTree()
    if kwargs:
        r = QRedis(**kwargs)
        window.add_redis(r)
    window.show()
    sys.exit(application.exec_())


if __name__ == "__main__":
    main()



from datetime import timedelta
from functools import partial

from .util import redis_str
from .qt import (
    Qt, QWidget, QMainWindow, QLabel, QStackedLayout, QMessageBox,
    QTableWidgetItem, QTreeWidgetItem, QIntValidator, ui_loadable)


ModifiedStyle = "background-color: rgb(255,200,200);"


@ui_loadable
class MultiEditor(QWidget):
    def __init__(self, parent=None):
        super(MultiEditor, self).__init__(parent)
        self.load_ui()
        self.modified = False
        self.item = None
        self.ui.table.itemSelectionChanged.connect(self.__update)
        self.ui.table.itemChanged.connect(self.__on_item_changed)
        self.ui.add_button.clicked.connect(self.__on_add_item)
        self.ui.delete_button.clicked.connect(self.__on_delete_selection)
        self.ui.revert_button.clicked.connect(self.__on_revert_changes)

    def __on_item_changed(self, item):
        self.modified = True
        self.__update()

    def __on_revert_changes(self):
        self.set_item(self.item)

    def __on_add_item(self):
        table = self.ui.table
        items = table.selectedIndexes()
        if items:
            row = max([item.row() for item in items])
        else:
            row = table.rowCount()
        table.insertRow(row)
        key_item = QTableWidgetItem()
        table.setItem(row, 0, key_item)
        table.editItem(key_item)
        self.modified = True
        self.__update()

    def __on_delete_selection(self):
        table = self.ui.table
        rows = set([item.row() for item in table.selectedIndexes()])
        for row in sorted(rows, reverse=True):
            table.removeRow(row)
        self.modified = True
        self.__update()

    def __update(self):
        ui, modified = self.ui, self.modified
        style = ModifiedStyle if modified else ""
        selected_count = len(ui.table.selectedIndexes())
        ui.table.setStyleSheet(style)
        ui.revert_button.setEnabled(modified)
        ui.delete_button.setEnabled(selected_count > 0)

    def get_item(self):
        table, item = self.ui.table, self.item
        if item.type == "hash":
            value = {}
            for row in range(table.rowCount()):
                value[table.item(row, 0).text()] = table.item(row, 1).text()
        else:
            value = [table.item(row, 0).text() for row in range(table.rowCount())]
            value = value if item.type == "list" else set(value)
        return item._replace(value=value)

    def set_item(self, item):
        self.item = item
        table = self.ui.table
        table.clearContents()
        dtype, value = item.type, item.value
        table.setRowCount(len(value))
        header = ("Key", "Value") if dtype == "hash" else ("Value",)
        table.setColumnCount(len(header))
        table.setHorizontalHeaderLabels(header)
        keys = value if dtype == "list" else sorted(value)
        for row, key in enumerate(keys):
            table.setItem(row, 0, QTableWidgetItem(key))
        if item.type == "hash":
            for row, key in enumerate(keys):
                table.setItem(row, 1, QTableWidgetItem(value[key]))
        self.modified = False
        self.__update()


@ui_loadable
class SimpleEditor(QWidget):
    def __init__(self, parent=None):
        super(SimpleEditor, self).__init__(parent)
        self.load_ui()
        self.__item = None
        self.__original_item = None
        ui = self.ui
        ui.key_value.textChanged.connect(self.__on_text_changed)
        ui.incr_button.clicked.connect(partial(self.__incr_by, scale=1))
        ui.decr_button.clicked.connect(partial(self.__incr_by, scale=-1))
        ui.revert_button.clicked.connect(self.__on_revert)

    def __on_revert(self):
        self.set_item(self.__original_item)

    def __on_text_changed(self):
        self.__item = self.__item._replace(value=self.ui.key_value.toPlainText())
        self.__update()

    def __update(self):
        ui = self.ui
        modified = self.modified
        ui.revert_button.setEnabled(modified)
        style = ModifiedStyle if modified else ""
        ui.key_value.setStyleSheet(style)

    def __incr_by(self, scale=1):
        step = self.ui.step_value.value() * scale
        text = self.ui.key_value.toPlainText()
        value = None
        try:
            value = int(text)
            step = int(step)
        except ValueError:
            try:
                value = float(text)
            except ValueError:
                pass
        if value is not None:
            self.ui.key_value.setPlainText(str(value + step))

    def get_item(self):
        return self.__item

    def set_item(self, item):
        self.__original_item = self.__item = item
        self.ui.key_value.setPlainText(item.value)

    @property
    def modified(self):
        return self.__original_item != self.__item


@ui_loadable
class RedisItemEditor(QWidget):
    def __init__(self, parent=None):
        super(RedisItemEditor, self).__init__(parent)
        self.load_ui()
        self.__original_item = None
        self.__item = None
        ui = self.ui
        layout = QStackedLayout(ui.type_editor)
        layout.setContentsMargins(3, 3, 3, 3)
        self.none_editor = QWidget()
        self.simple_editor = SimpleEditor()
        self.hash_editor = MultiEditor()
        self.seq_editor = MultiEditor()
        self.seq_editor.ui.table.horizontalHeader().setVisible(False)
        self.set_editor = MultiEditor()
        self.set_editor.ui.table.horizontalHeader().setVisible(False)
        layout.addWidget(self.none_editor)
        layout.addWidget(self.simple_editor)
        layout.addWidget(self.hash_editor)
        layout.addWidget(self.seq_editor)
        layout.addWidget(self.set_editor)
        self.type_editor_map = {
            "none": self.none_editor,
            "string": self.simple_editor,
            "hash": self.hash_editor,
            "list": self.seq_editor,
            "set": self.set_editor,
        }
        ttl_validator = QIntValidator()
        ttl_validator.setBottom(-1)
        ui.ttl_value.setValidator(ttl_validator)

        ui.key_name.textChanged.connect(self.__on_key_name_changed)
        ui.key_name.returnPressed.connect(self.__on_key_name_applied)
        ui.ttl_value.textChanged.connect(self.__on_ttl_changed)
        ui.ttl_value.returnPressed.connect(self.__on_ttl_applied)
        ui.apply_button.clicked.connect(self.__on_apply)
        ui.refresh_button.clicked.connect(self.__on_refresh)
        ui.persist_button.clicked.connect(self.__on_persist)
        ui.delete_button.clicked.connect(self.__on_delete)

    def __on_apply(self):
        editor = self.ui.type_editor.layout().currentWidget()
        item = editor.get_item()
        item = self.__item._replace(value=item.value)
        if self.__original_item.key:
            self.__on_key_name_applied()
        item.redis[item.key] = item.value
        self.__on_ttl_applied()
        self.set_item(item)
        self.__update()

    def __on_key_name_changed(self, key):
        self.__item = self.__item._replace(key=key)
        self.__update()

    def __on_key_name_applied(self):
        if self.name_modified:
            item, original_item = self.__item, self.__original_item
            try:
                if original_item.key:
                    item.redis.rename(original_item.key, item.key)
                    self.__original_item = original_item._replace(key=item.key)
            except Exception as e:
                import traceback

                traceback.print_exc()
                print("error", str(e))
            self.__update()

    def __on_ttl_changed(self, ttl):
        ttl = int(ttl) if ttl else -1
        self.__item = self.__item._replace(ttl=ttl)
        self.__update()

    def __on_ttl_applied(self):
        if self.ttl_modified:
            item, original_item = self.__item, self.__original_item
            try:
                item.redis.expire(item.key, item.ttl)
                self.__original_item = original_item._replace(ttl=item.ttl)
            except Exception as e:
                import traceback

                traceback.print_exc()
                print("error", str(e))
            self.__update()

    def __on_refresh(self):
        self.set_item(self.__item)

    def __on_persist(self):
        try:
            self.__item.redis.persist(self.__item.key)
        except Exception as e:
            print("error", str(e))

    def __on_delete(self):
        try:
            self.__item.redis.delete(self.__item.key)
        except Exception as e:
            print("error", str(e))

    def __update(self):
        ui = self.ui
        name_modified, ttl_modified = self.name_modified, self.ttl_modified
        ui.key_name.setStyleSheet(ModifiedStyle if name_modified else "")
        ui.ttl_value.setStyleSheet(ModifiedStyle if ttl_modified else "")
        if ttl_modified:
            ttl = self.__item.ttl
            ttl_str = "Persistent"
            if ttl > 0:
                ttl_str = "TTL: {0}".format(timedelta(seconds=ttl))
            ui.ttl_value.setToolTip(ttl_str)

    @property
    def name_modified(self):
        return self.__item.key != self.__original_item.key

    @property
    def ttl_modified(self):
        return self.__item.ttl != self.__original_item.ttl

    def set_item(self, item):
        if item is None:
            editor = self.none_editor
            ttl = -1
        else:
            editor = self.type_editor_map[item.type]
            editor.set_item(item)
            ttl = item.ttl
        self.__original_item = self.__item = item
        self.ui.type_editor.layout().setCurrentWidget(editor)
        if item is None:
            return
        self.ui.key_name.setText(item.key)
        self.ui.ttl_value.setText(str(ttl) if ttl > 0 else "")


@ui_loadable
class RedisDbEditor(QMainWindow):
    def __init__(self, parent=None):
        super(RedisDbEditor, self).__init__(parent)
        self.load_ui()
        ui = self.ui
        ui.name_label = QLabel("-----")
        ui.toolbar.addWidget(self.ui.name_label)
        ui.refresh_action.triggered.connect(self.__on_refresh)
        ui.info_filter.textChanged.connect(
            partial(self.__on_filter_changed, ui.info_table))
        ui.config_filter.textChanged.connect(
            partial(self.__on_filter_changed, ui.config_table))
        ui.config_table.itemChanged.connect(self.__on_config_changed)

    def __on_refresh(self):
        self.set_db(self._redis)

    def __on_filter_changed(self, table, text):
        if text:
            hidden = lambda item: not text in item.text()
        else:
            hidden = lambda item: False
        for row in range(table.rowCount()):
            item = table.item(row, 0)
            table.setRowHidden(row, hidden(item))

    def __on_config_changed(self, item):
        table = item.tableWidget()
        key = table.item(item.row(), 0).text()
        value = item.text()
        result = QMessageBox.question(
            self, "Are you sure?",
            "Are you sure you want to change the value of {} to {}?".format(key, value))
        if result != QMessageBox.Yes:
            return
        try:
            self._redis.config_set(key, value)
        except Exception as error:
            value = self._redis.config_get(key)[key]
            table.blockSignals(True)
            item.setText(value)
            table.blockSignals(False)
            QMessageBox.warning(self, "Error changing config", repr(error))


    def set_db(self, redis):
        self._redis = redis
        info = self._redis.info()
        config = self._redis.config_get()
        clients = self._redis.client_list()
        name, tooltip = redis_str(redis)
        name = "{} (v{})".format(name, info["redis_version"])
        self.ui.name_label.setText(name)
        self.ui.name_label.setToolTip(tooltip)

        # Info
        info_table = self.ui.info_table
        info_table.clearContents()
        info_table.setRowCount(len(info))
        for row, key in enumerate(sorted(info)):
            c0 = QTableWidgetItem(key)
            c0.setFlags(c0.flags() & (~Qt.ItemIsEditable))
            c1 = QTableWidgetItem(str(info[key]))
            c1.setFlags(c0.flags() & (~Qt.ItemIsEditable))
            info_table.setItem(row, 0, c0)
            info_table.setItem(row, 1, c1)

        # Config
        config_table = self.ui.config_table
        config_table.blockSignals(True)
        config_table.clearContents()
        config_table.setRowCount(len(config))
        for row, key in enumerate(sorted(config)):
            c0 = QTableWidgetItem(key)
            c0.setFlags(c0.flags() & (~Qt.ItemIsEditable))
            c1 = QTableWidgetItem(str(config[key]))
            c1.setFlags(c0.flags() |Qt.ItemIsEditable)
            config_table.setItem(row, 0, c0)
            config_table.setItem(row, 1, c1)
        config_table.blockSignals(False)

        # Clients
        ctree = self.ui.client_tree
        ctree.clear()
        for client in clients:
            name = str(client["id"])
            if client["name"]:
                name += " ({})".format(client["name"])
            item = QTreeWidgetItem(ctree, [name])
            for key, value in client.items():
                QTreeWidgetItem(item, [key, str(value)])


class RedisEditor(QWidget):
    def __init__(self, parent=None):
        super(RedisEditor, self).__init__(parent)
        self.empty = QWidget(self)
        self.item = RedisItemEditor(self)
        self.db = RedisDbEditor(self)
        self.db.setWindowFlags(Qt.Widget)
        layout = QStackedLayout(self)
        layout.addWidget(self.empty)
        layout.addWidget(self.item)
        layout.addWidget(self.db)

    def set_empty(self):
        self.layout().setCurrentWidget(self.empty)

    def set_item(self, item):
        self.layout().setCurrentWidget(self.item)
        self.item.set_item(item)

    def set_db(self, redis):
        self.layout().setCurrentWidget(self.db)
        self.db.set_db(redis)



from setuptools import setup, find_packages


def get_readme(name="README.md"):
    with open(name) as f:
        return f.read()


requirements = ["redis", "PyQt5"]


setup(
    name="qredis",
    version="0.3.0",
    description="Qt based Redis GUI",
    long_description=get_readme(),
    long_description_content_type="text/markdown",
    author="Tiago Coutinho",
    author_email="coutinhotiago@gmail.com",
    url="https://github.com/tiagocoutinho/qredis",
    packages=find_packages(),
    package_data={"qredis.images": ["*.png"], "qredis.ui": ["*.ui"]},
    entry_points={"console_scripts": ["qredis=qredis.window:main"]},
    install_requires=requirements,
    keywords="redis,GUI,Qt",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    python_requires=">=3.5",
)
